@@ -119,7 +119,7 @@ soup_queue_error_cb (gboolean body_started, gpointer user_data)
 	}
 }
 
-static SoupTransferDone
+static void
 soup_queue_read_headers_cb (const GString        *headers,
                             SoupTransferEncoding *encoding,
 			    gint                 *content_len,
@@ -223,15 +223,15 @@ soup_queue_read_headers_cb (const GString        *headers,
 
  SUCCESS_CONTINUE:
 	soup_message_run_handlers (req, SOUP_HANDLER_PRE_BODY);
-	return SOUP_TRANSFER_CONTINUE;
+	return;
 
  THROW_MALFORMED_HEADER:
 	soup_connection_set_keep_alive (req->connection, FALSE);
 	soup_message_issue_callback (req);
-	return SOUP_TRANSFER_END;
+	return;
 }
 
-static SoupTransferDone
+static void
 soup_queue_read_chunk_cb (const SoupDataBuffer *data,
 			  gpointer              user_data)
 {
@@ -243,7 +243,7 @@ soup_queue_read_chunk_cb (const SoupDataBuffer *data,
 
 	soup_message_run_handlers (req, SOUP_HANDLER_BODY_CHUNK);
 
-	return SOUP_TRANSFER_CONTINUE;
+	return;
 }
 
 static void
@@ -258,6 +258,8 @@ soup_queue_read_done_cb (const SoupDataBuffer *data,
 	req->response.length = data->length;
 	req->response.body = data->body;
 
+	soup_transfer_read_unref (req->priv->read_tag);
+
 	if (req->errorclass == SOUP_ERROR_CLASS_INFORMATIONAL) {
 		GIOChannel *channel;
 		gboolean overwrt;
@@ -278,7 +280,7 @@ soup_queue_read_done_cb (const SoupDataBuffer *data,
 	} 
 	else {
 		req->status = SOUP_STATUS_FINISHED;
-		req->priv->read_tag = 0;
+		req->priv->read_tag = NULL;
 	}
 
 	soup_message_run_handlers (req, SOUP_HANDLER_POST_BODY);
@@ -293,23 +295,22 @@ soup_encode_http_auth (SoupMessage *msg, GString *header, gboolean proxy_auth)
 
 	ctx = proxy_auth ? soup_get_proxy () : msg->context;
 
-	if (msg->connection->auth)
-		auth = msg->connection->auth;
-	else
-		auth = soup_auth_lookup (ctx);
+	auth = soup_context_lookup_auth (ctx, msg);
+	if (!auth)
+		return;
+	if (!auth->authenticated &&
+	    !soup_context_authenticate_auth (ctx, auth))
+		return;
 
-	if (auth) {
-		token = soup_auth_authorize (auth, msg);
-		if (token) {
-			g_string_sprintfa (header, 
-					   "%s: %s\r\n",
-					   proxy_auth ? 
-					   	"Proxy-Authorization" : 
-					   	"Authorization",
-					   token);
-			g_free (token);
-		}
- 	}
+	token = soup_auth_authorize (auth, msg);
+	if (token) {
+		g_string_sprintfa (header, "%s: %s\r\n",
+				   proxy_auth ? 
+					"Proxy-Authorization" : 
+					"Authorization",
+				   token);
+		g_free (token);
+	}
 }
 
 struct SoupUsedHeaders {
@@ -462,7 +463,8 @@ soup_queue_write_done_cb (gpointer user_data)
 {
 	SoupMessage *req = user_data;
 
-	req->priv->write_tag = 0;
+	soup_transfer_write_unref (req->priv->write_tag);
+	req->priv->write_tag = NULL;
 	req->status = SOUP_STATUS_READING_RESPONSE;
 }
 
