@@ -175,7 +175,8 @@ soup_server_unref (SoupServer *serv)
 					     serv);
 		g_hash_table_destroy (serv->handlers);
 
-		g_main_destroy (serv->loop);
+		if (serv->loop)
+			g_main_loop_unref (serv->loop);
 
 		g_free (serv);
 	}
@@ -293,7 +294,7 @@ destroy_message (SoupMessage *msg)
 	 * If CGI, service one message and quit 
 	 */
 	if (server->proto == SOUP_PROTOCOL_CGI)
-		g_main_quit (server->loop);
+		g_main_loop_quit (server->loop);
 
 	soup_server_unref (server);
 
@@ -314,6 +315,7 @@ write_done_cb (gpointer user_data)
 {
 	SoupMessage *msg = user_data;
 
+	soup_transfer_write_unref (msg->priv->write_tag);
 	msg->priv->write_tag = 0;
 	destroy_message (msg);
 }
@@ -542,7 +544,7 @@ issue_bad_request (SoupMessage *msg)
 	g_io_channel_unref (channel);
 } /* issue_bad_request */
 
-static SoupTransferDone
+static void
 read_headers_cb (const GString        *headers,
 		 SoupTransferEncoding *encoding,
 		 gint                 *content_len,
@@ -655,14 +657,12 @@ read_headers_cb (const GString        *headers,
 
 	g_free (req_path);
 
-	return SOUP_TRANSFER_CONTINUE;
+	return;
 
  THROW_MALFORMED_HEADER:
 	g_free (req_path);
 
 	issue_bad_request(msg);
-
-	return SOUP_TRANSFER_CONTINUE;
 }
 
 static void
@@ -863,6 +863,7 @@ read_done_cb (const SoupDataBuffer *data,
 	SoupSocket *server_sock = req->priv->server_sock;
 	GIOChannel *channel;
 
+	soup_transfer_read_unref (req->priv->read_tag);
 	req->priv->read_tag = 0;
 
 	call_handler (req, data, soup_context_get_uri (req->context)->path);
@@ -1133,7 +1134,7 @@ soup_server_run_async (SoupServer *server)
 
  START_ERROR:
 	if (server->loop) {
-		g_main_destroy (server->loop);
+		g_main_loop_unref (server->loop);
 		server->loop = NULL;
 	}
 }
@@ -1144,12 +1145,12 @@ soup_server_run (SoupServer *server)
 	g_return_if_fail (server != NULL);
 
 	if (!server->loop) {
-		server->loop = g_main_new (TRUE);
+		server->loop = g_main_loop_new (NULL, TRUE);
 		soup_server_run_async (server);
 	}
 
 	if (server->loop)
-		g_main_run (server->loop);
+		g_main_loop_run (server->loop);
 }
 
 void 
@@ -1157,7 +1158,7 @@ soup_server_quit (SoupServer *server)
 {
 	g_return_if_fail (server != NULL);
 
-	g_main_quit (server->loop);
+	g_main_loop_quit (server->loop);
 	soup_server_unref (server);
 }
 
